/*
* AVR Binary Clock for the 8515
*
* Danny H. Newport
* BinClock 8515 using 1hz input on D2 and 60hz input on D3
* compiled with AvrEdit Ver. 3.5 using 61% memory (room for expansion)
*
*
*   This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   
*
*
*	Special Thanks.
*		AVR GCC team for a great compiler
*		AvrEdit for a great avr ide setup
*		Atmel for the versitile Micro.    
* 
*/

#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>
#include <timer.h>
#define __AVR_AT90S8515__

//defines for delay
#define CPU_freq    10000000						//cpu clock frequency
#define CYCLES_PER_US ((CPU_freq+500000)/1000000) 	// cpu cycles per microsecond
#undef ECHO
#define UART_BAUD_RATE 9600
#define UART_BAUD_SELECT ((CPU_freq/(UART_BAUD_RATE*16))-1)

typedef unsigned char  u08;
/* global variables used by PWM outputs */
/*volatile uint8_t pwm[8];
volatile uint8_t pwm_config;
typedef unsigned char  u08;*/
u08 thour;
u08 hour;
u08 tmin;
u08 min;
u08 tsec;
u08 timeprefInt;
u08 sec;
u08 loop;
u08 display[20]={0};
u08 seconds=00;
u08 minutes=0;
u08 hours=12;
//int led[20]={PINA4,PINA5,PINA6,PINA7,PINA1,PINA2,PINA3,PINB4,PINB5,PINB6,PINB7,PINB1,PINB2,PINB3,PINC3,PINC4,PINC5,PINC1,PINC2,PINC6};
int led[19]={PINA0,PINA1,PINA2,PINA3,PINA4,PINA5,PINA6,PINB0,PINB1,PINB2,PINB3,PINB4,PINB5,PINB6,PINC0,PINC1,PINC2,PINC3,PINC4};
/* uart globals */
/*static volatile u08 *uart_data_ptr;
static volatile u08 uart_counter;*/

/* global variables for delay functions */
uint16_t delay_count=CPU_freq/4000;				// used in delay function

/* prototypes */
//void dobuffer (void);

/* global variables for serial port */
//static uint8_t recv_bytes = 0;
//static char buffer[10];
uint8_t m_dtmf;

/* device ID (can be any char) */
#define ID '0'
/* timer 0 initial count */
/* 0 -> 25,6 us PWM resolution @ 10 Mhz */
/* 56 -> 20 us */
#define STARTCNT0 56

/* global variables for serial port */
static char buffer[10];
volatile uint8_t ready = 0;
char buf[]="Hello World!!!";

uint8_t m_time=0;
uint8_t m_found=0;
uint8_t m_enable=0;
#define FALSE               0
#define TRUE                1

#define DDR_PORT_INPUT      ( 0 )
#define DDR_PORT_OUTPUT     ( 0xff )
#define IN_PORT_TRIS        ( 0 )
#define IN_PORT_PULL        ( 0xff )

#define STARTUP_DELAY       ( 30U )

typedef uint8_t BOOL;

/* global variables for serial port */
static uint8_t recv_bytes = 0;
static char buffer[10];
//volatile uint8_t ready = 0;

/* prototypes */
void dobuffer (void);


void extint_init(void)
{
	//{{WIZARD_MAP(External IRQ)
	// INT1 Enabled, Mode: Low Level
	//	MCUCR |= 0x0;
	//GIMSK |= 0x80;
	//}}WIZARD_MAP(External IRQ)
}
/*
SIGNAL(SIG_INTERRUPT1)
{
// TODO: Add your code here
//DoTimeDisplay();
}*/
/*
* Set default values
*/
void init(void)
{
	thour=0;
	hour=2;
	tmin=2;
	min=23;
	tsec=0;
	sec=0;
	loop=0;
	timeprefInt=12;

	// Timer/Counter0 Clock source: System Clock
	// Timer/Counter0 Clock value: Stopped
	//	TCNT0 = 0x0;
	//	TCCR0 = 0x0;

	// Timer/Counter1 Clock source: System Clock
	// Timer/Counter1 Clock value: 39.063kHz
	// Timer/Counter1 Mode: Normal
	// Timer/Counter1 Output: A: Disconnected, B: Disconnected
	/*	OCR1A = 0x9896;
	OCR1B = 0x9896;
	TCNT1 = 0x676a;
	TCCR1A = 0x0;
	TCCR1B = 0x4;

	TIMSK = 0x0;
	extint_init();
	*/
}

/*
*
*/
void
pause (void)
{
	while (!ready);
	ready = 0;
}


/*
* uart transmit complete isr
*/
INTERRUPT (SIG_UART_TRANS)
{
	ready = 1;
}

/*
*
*/
void
receive_loop (void)
{
	uint8_t b;

	for (;;)
	{
		//loop_until_bit_is_set (USR, 7);	/* wait for serial data */
		b = inp (UDR);		/* read it */
		buffer[recv_bytes++] = b;
#ifdef ECHO
		outp (b, UDR);
#endif
		if ((recv_bytes == 10) || (b < 32))
		{
			dobuffer ();
			recv_bytes = 0;
		}
	}
}

void
delay ( uint16_t delay ) {

	uint16_t counter = 0;

	for ( ; counter < delay; counter++ );

}


void
delay_factor ( uint16_t delay_factor ) {

	while ( delay_factor-- != 0 ) {
		delay( 25 );
	}

}



/*
*
*/
void
dobuffer (void)
{
	/*  uint8_t pos=0;
	//uint8_t tmpq1, pwmtoset, result;
	char *ptr;

	ptr = &buffer[0];
	lcd_set_display_address( DA_ARGS_LINE1 );

	for(pos=0;pos<recv_bytes;pos++)
	{
	lcd_put_char(buffer[pos]);
	pause();
	}
	outp('x',UDR);
	outp('\n',UDR);*/
	return;

}




/*
* Convert an ASCII char to a 4 bits value
*/
uint8_t quartet (char c)
{
	/* to upper case */
	if (c >= 'a')
		c = c - 32;
	if (c < 'A')
		return (c - '0');
	else
		return (c - 'A' + 10);
}

char tobcd (uint8_t i)
{
	if (i > 9)
		return (i + 'A' - 10);
	else
		return (i + '0');
}




// millisecond delay function
void delay_ms(uint8_t ms)
{
	uint16_t cnt; asm volatile (

		"\n" "L_dl1%=:" "\n\t" "mov %A0, %A2" "\n\t" "mov %B0, %B2" "\n" "L_dl2%=:" "\n\t"

		"sbiw %A0, 1" "\n\t" "brne L_dl2%=" "\n\t" "dec %1" "\n\t" "brne L_dl1%=" "\n\t" : "=&w" (cnt) : "r" (ms), "r" (delay_count) );
}
void DoTimeDisplay(void)
{

	
	if(bit_is_clear(PIND,PIND3)==1)//60HZ INPUT
		m_enable=1;					//enable leds
	else m_enable=0 ;				//disable leds

	if(seconds>59)
	{
		seconds=0;
		minutes++;
	}
	if(minutes>59)
	{
		minutes=0;
		hours++;
	}
	if(hours>12)
	{
		hours=1;
	}
	sec=seconds;
	min=minutes;
	hour=hours;

	if(sec<10)
		tsec = 0;
	if(sec>9&&sec<20)
	{
		tsec = 1;
		sec = sec-10;
	}
	if(sec>19&&sec<30)
	{

		tsec = 2;
		sec = sec-20;
	}

	if(sec>29&&sec<40)
	{
		tsec = 3;
		sec = sec-30;
	}
	if (sec>39&&sec<50)
	{
		tsec = 4;
		sec = sec-40;
	}
	if(sec>49)
	{
		tsec = 5;
		sec = sec-50;
	}


	if(min<10)
		tmin = 0;
	if(min>9&&min<20)
	{
		tmin = 1;
		min = min-10;
	}
	if(min>19&&min<30)
	{

		tmin = 2;
		min = min-20;
	}

	if(min>29&&min<40)
	{
		tmin = 3;
		min = min-30;
	}
	if (min>39&&min<50)
	{
		tmin = 4;
		min = min-40;
	}
	if(min>49)
	{
		tmin = 5;
		min = min-50;
	}



	if(hour==0 && timeprefInt==12) //if hour is 0 and prefs are 12hr, then change hour to 12
		hour=12;

	if(hour>=13)
		hour=hour - timeprefInt;

	if(hour<10)
		thour = 0;
	if(hour>9&&hour<20)
	{
		thour = 1;
		hour = hour-10;
	}
	if(hour>19&&hour<30)
	{

		thour = 2;
		hour = hour-20;
	}


	for(loop=0;loop<20;loop++)
	{
		display[loop]=0;
	}

	if(sec>7&&m_enable)
	{
		display[3]=1;

		sbi(PORTA,PINA3);
	}
	else cbi(PORTA,PINA3);
	if((sec>3&&sec<8)&&m_enable)
	{
		display[2]=1;
		sbi(PORTA,PINA2);
	}
	else cbi(PORTA,PINA2);
	if((sec==2||sec==3||sec==6||sec==7)&&m_enable)
	{
		display[1]=1;
		sbi(PORTA,PINA1);
	}
	else cbi(PORTA,PINA1);
	if((sec==1||sec==9||sec==3||sec==5||sec==7)&&m_enable)
	{
		display[0]=1;
		sbi(PORTA,PINA0);
	}
	else cbi(PORTA,PINA0);



	if((tsec>3&&tsec<8)&&m_enable)
	{
		display[6]=1;
		sbi(PORTA,PINA6);
	}else cbi(PORTA,PINA6);
	if((tsec==2||tsec==3||tsec==6||tsec==7)&&m_enable)
	{
		display[5]=1;
		sbi(PORTA,PINA5);
	}else cbi(PORTA,PINA5);
	if(tsec==1||tsec==9||tsec==3||tsec==5||tsec==7)
	{
		display[4]=1;
		sbi(PORTA,PINA4);
	}else cbi(PORTA,PINA4);




	if(min>7&&m_enable)
	{
		display[10]=1;
		sbi(PORTB,PINB3);
	}else cbi(PORTB,PINB3);
	if((min>3&&min<8)&&m_enable)
	{
		display[9]=1;
		sbi(PORTB,PINB2);
	}else cbi(PORTB,PINB2);
	if((min==2||min==3||min==6||min==7)&&m_enable)
	{
		display[8]=1;
		sbi(PORTB,PINB1);
	}else cbi(PORTB,PINB1);
	if((min==1||min==9||min==3||min==5||min==7)&&m_enable)
	{
		display[7]=1;
		sbi(PORTB,PINB0);
	}else cbi(PORTB,PINB0);


	if(tmin>3&&tmin<8)
	{
		display[13]=1;
		sbi(PORTB,PINB6);
	}else cbi(PORTB,PINB6);
	if((tmin==2||tmin==3||tmin==6||tmin==7)&&m_enable)
	{
		display[12]=1;
		sbi(PORTB,PINB5);
	}else cbi(PORTB,PINB5);
	if((tmin==1||tmin==9||tmin==3||tmin==5||tmin==7)&&m_enable)
	{
		display[11]=1;
		sbi(PORTB,PINB4);
	}else cbi(PORTB,PINB4);

	if(hour>7&&m_enable)
	{
		display[17]=1;
		sbi(PORTC,PINC5);
	}else cbi(PORTC,PINC5);
	if((hour>3&&hour<8)&&m_enable)
	{
		display[16]=1;
		sbi(PORTC,PINC2);
	}else cbi(PORTC,PINC2);
	if((hour==2||hour==3||hour==6||hour==7)&&m_enable)
	{
		display[15]=1;
		sbi(PORTC,PINC1);
	}else cbi(PORTC,PINC1);
	if((hour==1||hour==9||hour==3||hour==5||hour==7)&&m_enable)
	{
		display[14]=1;

		sbi(PORTC,PINC0);
	}else cbi(PORTC,PINC0);



	if((thour>1&&thour<3)&&m_enable)
	{
		display[19]=1;

		sbi(PORTC,PINC4);
	}else cbi(PORTC,PINC4);
	if(thour==1&&m_enable)
	{
		display[18]=1;

		sbi(PORTC,PINC3);
	}else cbi(PORTC,PINC3);


}
/*
*  function for endless loop
*/
void main_loop (void)
{
//	sbi(PORTA,PINA2);
//	sbi(PORTD,PIND4);
//	sbi(PORTC,PINC6);
//	sbi(PORTC,PINC7);

	for (;;)
	{
		while(bit_is_clear(PINC,PINC6)==1)//hold button
		{
			/*         cbi(PORTA,PINA0);
			cbi(PORTA,PINA1);
			sbi(PORTA,PINA2);
			cbi(PORTA,PINA3);
			*/
			DoTimeDisplay();
			//DONOTHING BUT HOLD
		}
		if(bit_is_clear(PINC,PINC7)==1)//fast forward button
		{
			/*          sbi(PORTA,PINA0);
			cbi(PORTA,PINA1);
			cbi(PORTA,PINA2);
			cbi(PORTA,PINA3);
			*/
			seconds+=1;
			DoTimeDisplay();
			//  for(loop=0;loop<100;loop++)
			delay_ms(1);

		}else if(bit_is_clear(PIND,PIND4)==1)//slow forward button
		{
			//       cbi(PORTA,PINA0);
			//     cbi(PORTA,PINA1);
			//   cbi(PORTA,PINA2);
			// sbi(PORTA,PINA3);
			for(loop=0;loop<5;loop++)
				delay_ms(2);
			seconds+=1;
			DoTimeDisplay();

		}else           //just do the time
		{

			if(bit_is_clear(PIND,PIND2)==1)//pin d2 is a 1 sec timer input from ELM440 chip
			{

				seconds+=1;
				DoTimeDisplay();
				while(bit_is_clear(PIND,PIND2)==1)
				{

					DoTimeDisplay();


				}
			}

			DoTimeDisplay();




		}

	}

}



/*
*
*/
int main (void)
{
	init();
	/* 10Mhz / 200 = 1 overflow every 20,0 us @ 10MHz */
	/* 255 steps = 5.1 ms */
	outp (0xff, DDRA);
	outp (0xff, DDRB);			/* portb = input */
	outp (0xff, DDRC);
	outp (0x00, DDRD);
	outp ((1 << TXCIE) | (1 << RXEN) | (1 << TXEN), UCR);
	outp (64, UBRR);


	sei ();						/* enable interrupts */
	main_loop ();
	return 0 ;
}



